ZoomBasicWindow=class(Turbine.UI.Window)
-- NOTE width and height are SET using actual scale and RETURN with zoomed scale - that way we can use normal layout calculations at the zoomed level but set positioning and size based on actual size
-- to get the actual width and height you must externally mulitply by ZoomBasicWindow.Zoom (such as for saving size for plugindata, etc.

function ZoomBasicWindow:Constructor()
	Turbine.UI.Window.Constructor(self)
	self.Zoom=1
	self.Resizable=false
	self.Width,self.Height=Turbine.UI.Window.GetSize(self) -- called before overridden so we get the default size
	self.ActualWidth,self.ActualHeight=self:GetSize() -- we track the actual size values to avoid rounding issues
end
function ZoomBasicWindow:SetResizable(resizable)
	self.Resizable=resizable
end
function ZoomBasicWindow:IsResizable()
	return self.Resizable
end
function ZoomBasicWindow:SetZoom(zoom)
	self.Zoom=zoom
	if zoom>1 then
		-- disable built in zooming
--		self:SetStretchMode(1)
	else
--		self:SetStretchMode(0)
	end
	self:SetSize(self:GetActualWidth(),self:GetActualHeight())
	if self.SizeChanged~=nil then self.SizeChanged(self) end -- fire the sizechanged eventhandler to allow re-laying out controls even though overall size may not have actually changed (the zoomed controls will have changed)
end
function ZoomBasicWindow:GetZoom()
	return self.Zoom
end
function ZoomBasicWindow:GetWidth()
	return self.Width
end
function ZoomBasicWindow:GetActualWidth()
	return self.ActualWidth
end
function ZoomBasicWindow:GetHeight()
	return self.Height
end
function ZoomBasicWindow:GetActualHeight()
	return self.ActualHeight
end
function ZoomBasicWindow:GetSize()
	return self.Width, self.Height
end
function ZoomBasicWindow:GetActualSize()
	return self.ActualWidth,self.ActualHeight
end
function ZoomBasicWindow:SetWidth(width)
	self:SetSize(width,inventoryWindow.ActualHeight)
end
function ZoomBasicWindow:SetHeight(height)
	self:SetSize(inventoryWindow.ActualWidth,height)
end
function ZoomBasicWindow:SetSize(width,height)
	if width<self:GetMinimumWidth() then width=self:GetMinimumWidth() end
	if height<self:GetMinimumHeight() then height=self:GetMinimumHeight() end
	self:SetStretchMode(0)
	Turbine.UI.Window.SetSize(self,width/self.Zoom,height/self.Zoom)
	self:SetStretchMode(1)
	self.ActualWidth=width
	self.ActualHeight=height
	self.Width=width/self.Zoom
	self.Height=height/self.Zoom
	Turbine.UI.Window.SetSize(self,width,height)
end

--************* functions to allow resizing when Resizable==true and shift, control or alt are pressed
ZoomBasicWindow.MouseDown=function(sender,args)
	if sender.Resizable and (sender.IsAltKeyDown() or sender.IsControlKeyDown() or sender.IsShiftKeyDown()) then
		sender.Sizing=true
		sender.SizeStartX=sender.ActualWidth
		sender.SizeDeltaX=args.X
		sender.SizeStartY=sender.ActualHeight
		sender.SizeDeltaY=args.Y
	end
end
ZoomBasicWindow.MouseUp=function(sender,args)
	sender.Sizing=false
end
ZoomBasicWindow.MouseMove=function(sender,args)
	if sender.Sizing then
		sender:SetSize(sender.SizeStartX+(args.X-sender.SizeDeltaX)*sender.Zoom,sender.SizeStartY+(args.Y-sender.SizeDeltaY)*sender.Zoom)
	end
end