ZoomWindow=class(Turbine.UI.Lotro.Window)
-- NOTE width and height are SET using actual scale and RETURN with zoomed scale - that way we can use normal layout calculations at the zoomed level but set positioning and size based on actual size
-- to get the actual width and height you must externally mulitply by ZoomWindow.Zoom (such as for saving size for plugindata, etc.

function ZoomWindow:Constructor()
	Turbine.UI.Lotro.Window.Constructor(self)
	self.Zoom=1
	self.Resizable=false
	self.Width,self.Height=Turbine.UI.Lotro.Window.GetSize(self) -- called before overridden so we get the default size
	self.ActualWidth,self.ActualHeight=self:GetSize() -- we track the actual size values to avoid rounding issues
end
function ZoomWindow:SetResizable(resizable)
	self.Resizable=resizable
	if self.Zoom>1 then
		-- disable built in sizing, we handle this manually
		Turbine.UI.Lotro.Window.SetResizable(self,false)
	else
		Turbine.UI.Lotro.Window.SetResizable(self,resizable)
	end
end
function ZoomWindow:IsResizable()
	return self.Resizable
end
function ZoomWindow:SetZoom(zoom)
	self.Zoom=zoom
	if zoom>1 then
		-- disable built in zooming
		Turbine.UI.Lotro.Window.SetResizable(self,false)
	else
		Turbine.UI.Lotro.Window.SetResizable(self,self:IsResizable())
--		Turbine.UI.Lotro.Window.SetStretchMode(self,0)
	end
	self:SetSize(self:GetActualWidth(),self:GetActualHeight())
	if self.SizeChanged~=nil then self.SizeChanged(self) end -- fire the sizechanged eventhandler to allow re-laying out controls even though overall size may not have actually changed (the zoomed controls will have changed)
end
function ZoomWindow:GetZoom()
	return self.Zoom
end
function ZoomWindow:GetWidth()
	return self.Width
end
function ZoomWindow:GetActualWidth()
	return self.ActualWidth
end
function ZoomWindow:GetHeight()
	return self.Height
end
function ZoomWindow:GetActualHeight()
	return self.ActualHeight
end
function ZoomWindow:GetSize()
	return self.Width, self.Height
end
function ZoomWindow:GetActualSize()
	return self.ActualWidth,self.ActualHeight
end
function ZoomWindow:SetWidth(width)
	self:SetSize(width,inventoryWindow.ActualHeight)
end
function ZoomWindow:SetHeight(height)
	self:SetSize(inventoryWindow.ActualWidth,height)
end
function ZoomWindow:SetSize(width,height)
	if width<self:GetMinimumWidth() then width=self:GetMinimumWidth() end
	if height<self:GetMinimumHeight() then height=self:GetMinimumHeight() end
	if self:GetZoom()>1 then
		self:SetStretchMode(0)
		Turbine.UI.Lotro.Window.SetSize(self,width/self.Zoom,height/self.Zoom)
		self:SetStretchMode(1)
	end
	self.ActualWidth=width
	self.ActualHeight=height
	self.Width=width/self.Zoom
	self.Height=height/self.Zoom
	Turbine.UI.Lotro.Window.SetSize(self,width,height)
end

--************* functions to allow resizing when Resizable==true and shift, control or alt are pressed
-- due to issues with the mouse not interacting quite right with the zoomed border for resizing, we enable resizing with a Shift+drag or Alt+drag
ZoomWindow.MouseDown=function(sender,args)
	if sender.Resizable and (sender.IsAltKeyDown() or sender.IsControlKeyDown() or sender.IsShiftKeyDown()) then
		sender.Sizing=true
		sender.SizeStartX=sender.ActualWidth
		sender.SizeDeltaX=args.X
		sender.SizeStartY=sender.ActualHeight
		sender.SizeDeltaY=args.Y
	end
end
ZoomWindow.MouseUp=function(sender,args)
	sender.Sizing=false
end
ZoomWindow.MouseMove=function(sender,args)
	if sender.Sizing then
		sender:SetSize(sender.SizeStartX+(args.X-sender.SizeDeltaX)*sender.Zoom,sender.SizeStartY+(args.Y-sender.SizeDeltaY)*sender.Zoom)
	end
end